/*************************************************************************

    File: englishGrammar.pl
    Copyright (C) 2004,2005,2006 Patrick Blackburn & Johan Bos

    This file is part of BB1, version 1.3 (November 2006).

    Modified by Frank Richter, May 2008 (parse tree output)

    BB1 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    BB1 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BB1; if not, write to the Free Software Foundation, Inc., 
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*************************************************************************/

/*========================================================================
   Texts
========================================================================*/

t(disc(S_st),[sem:T])--> 
   s(S_st,[coord:no,sem:S]),
   {combine(t:T,[s:S])}.

t(disc(S_st),[sem:T])--> 
   s(S_st,[coord:yes,sem:S]),
   {combine(t:T,[s:S])}.

t(disc(Q_st),[sem:T])--> 
   q(Q_st,[sem:Q]),
   {combine(t:T,[q:Q])}.


/*========================================================================
   Sentences
========================================================================*/

s(s(NP_st,VP_st),[coord:no,sem:Sem])--> 
   np(NP_st,[coord:_,num:Num,gap:[],sem:NP]), 
   vp(VP_st,[coord:_,inf:fin,num:Num,gap:[],sem:VP]), 
   {combine(s:Sem,[np:NP,vp:VP])}.

s(s(S1_st,S2_st),[coord:yes,sem:Sem])--> 
   s(S1_st,[coord:ant,sem:S1]), 
   s(S2_st,[coord:con,sem:S2]), 
   {combine(s:Sem,[s:S1,s:S2])}.

s(s(S1_st,S2_st),[coord:yes,sem:Sem])--> 
   s(S1_st,[coord:either,sem:S1]), 
   s(S2_st,[coord:or,sem:S2]), 
   {combine(s:Sem,[s:S1,s:S2])}.

s(s(if,S_st),[coord:ant,sem:Sem])--> 
   [if], 
   s(S_st,[coord:no,sem:S]),
   {combine(s:Sem,[if:S])}.

s(s(either,S_st),[coord:either,sem:Sem])--> 
   [either], 
   s(S_st,[coord:no,sem:S]),
   {combine(s:Sem,[either:S])}.

s(s(then,S_st),[coord:con,sem:Sem])--> 
   [then], 
   s(S_st,[coord:no,sem:S]),
   {combine(s:Sem,[then:S])}.

s(s(S_st),[coord:con,sem:Sem])-->
   s(S_st,[coord:no,sem:S]),
   {combine(s:Sem,[then:S])}.

s(s(or,S_st),[coord:or,sem:Sem])-->
   [or], 
   s(S_st,[coord:no,sem:S]),
   {combine(s:Sem,[or:S])}.

sinv(vp(AUX_st,NP_st,VP_st),[gap:G,sem:S])-->
   av(AUX_st,[inf:fin,num:Num,sem:Sem]),
   np(NP_st,[coord:_,num:Num,gap:[],sem:NP]),
   vp(VP_st,[coord:_,inf:inf,num:Num,gap:G,sem:VP]), 
   {combine(sinv:S,[av:Sem,np:NP,vp:VP])}.


/*========================================================================
   Questions
========================================================================*/

q(s(Wh_st,VP_st),[sem:Sem])--> 
   whnp(Wh_st,[num:Num,sem:NP]), 
   vp(VP_st,[coord:_,inf:fin,num:Num,gap:[],sem:VP]), 
   {combine(q:Sem,[whnp:NP,vp:VP])}.

q(s(Wh_st,VP_st),[sem:Sem])--> 
   whnp(Wh_st,[num:_,sem:NP]), 
   sinv(VP_st,[gap:[np:NP],sem:S]),
   {combine(q:Sem,[sinv:S])}.


/*========================================================================
   Noun Phrases
========================================================================*/

np(np(t),[coord:no,num:sg,gap:[np:NP],sem:NP])--> [].

np(np(NP1_st,Coord_st,NP2_st),[coord:yes,num:pl,gap:[],sem:NP])--> 
   np(NP1_st,[coord:no,num:sg,gap:[],sem:NP1]), 
   coord(Coord_st,[type:conj,sem:C]), 
   np(NP2_st,[coord:_,num:_,gap:[],sem:NP2]), 
   {combine(np:NP,[np:NP1,coord:C,np:NP2])}.

np(np(NP1_st,Coord_st,NP2_st),[coord:yes,num:sg,gap:[],sem:NP])--> 
   np(NP1_st,[coord:no,num:sg,gap:[],sem:NP1]), 
   coord(Coord_st,[type:disj,sem:C]), 
   np(NP2_st,[coord:_,num:sg,gap:[],sem:NP2]), 
   {combine(np:NP,[np:NP1,coord:C,np:NP2])}.

np(np(Det_st,N1_st),[coord:no,num:sg,gap:[],sem:NP])--> 
   det(Det_st,[mood:decl,type:_,sem:Det]), 
   n(N1_st,[coord:_,sem:N]), 
   {combine(np:NP,[det:Det,n:N])}.

np(np(PN_st),[coord:no,num:sg,gap:[],sem:NP])--> 
   pn(PN_st,[sem:PN]), 
   {combine(np:NP,[pn:PN])}.

np(np(Qnp_st),[coord:no,num:sg,gap:[],sem:NP])--> 
   qnp(Qnp_st,[mood:decl,sem:QNP]), 
   {combine(np:NP,[qnp:QNP])}.


/*========================================================================
   WH Noun Phrases
========================================================================*/

whnp(wh_np(N1_st),[num:sg,sem:NP])--> 
   qnp(N1_st,[mood:int,sem:QNP]), 
   {combine(whnp:NP,[qnp:QNP])}.

whnp(wh_np(Det_st,N1_st),[num:sg,sem:NP])--> 
   det(Det_st,[mood:int,type:_,sem:Det]), 
   n(N1_st,[coord:_,sem:N]), 
   {combine(whnp:NP,[det:Det,n:N])}.


/*========================================================================
   Nouns
========================================================================*/

n(n1(N1_st,Coord_st,N2_st),[coord:yes,sem:N])--> 
   n(N1_st,[coord:no,sem:N1]), 
   coord(Coord_st,[type:_,sem:C]),  
   n(N2_st,[coord:_,sem:N2]),
   {combine(n:N,[n:N1,coord:C,n:N2])}.

n(n1(A_st,N_st),[coord:C,sem:Sem])--> 
   adj(A_st,[sem:A]), 
   n(N_st,[coord:C,sem:N]), 
   {combine(n:Sem,[adj:A,n:N])}.

n(n1(N_st),[coord:no,sem:N])--> 
   noun(N_st,[sem:Noun]),
   {combine(n:N,[noun:Noun])}.

n(n1(N_st,Nmod_st),[coord:no,sem:Sem])--> 
   noun(N_st,[sem:N]), 
   nmod(Nmod_st,[sem:PP]),
   {combine(n:Sem,[noun:N,nmod:PP])}. 

nmod(nmod(PP_st),[sem:N])--> 
   pp(PP_st,[sem:PP]),
   {combine(nmod:N,[pp:PP])}.

nmod(nmod(RC_st),[sem:N])--> 
   rc(RC_st,[sem:RC]),
   {combine(nmod:N,[rc:RC])}.

nmod(nmod(PP_st,Nmod_st),[sem:Sem])--> 
   pp(PP_st,[sem:PP]), 
   nmod(Nmod_st,[sem:NMod]),
   {combine(nmod:Sem,[pp:PP,nmod:NMod])}.


/*========================================================================
   Verb Phrases
========================================================================*/

vp(vp(VP1_st,Coord_st,VP2_st),[coord:yes,inf:Inf,num:Num,gap:[],sem:VP])--> 
   vp(VP1_st,[coord:no,inf:Inf,num:Num,gap:[],sem:VP1]), 
   coord(Coord_st,[type:_,sem:C]), 
   vp(VP2_st,[coord:_,inf:Inf,num:Num,gap:[],sem:VP2]),
   {combine(vp:VP,[vp:VP1,coord:C,vp:VP2])}.

vp(vp(AUX_st,VP_st),[coord:no,inf:Inf,num:Num,gap:[],sem:VP])--> 
   av(AUX_st,[inf:Inf,num:Num,sem:Mod]), 
   vp(VP_st,[coord:_,inf:inf,num:Num,gap:[],sem:V2]), 
   {combine(vp:VP,[av:Mod,vp:V2])}.

vp(vp(AUX_st,NP_st),[coord:no,inf:Inf,num:Num,gap:[],sem:VP])--> 
   cop(AUX_st,[inf:Inf,num:Num,sem:Cop]), 
   np(NP_st,[coord:_,num:_,gap:[],sem:NP]), 
   {combine(vp:VP,[cop:Cop,np:NP])}.

vp(vp(V_st),[coord:no,inf:Inf,num:Num,gap:[],sem:VP])--> 
   iv(V_st,[inf:Inf,num:Num,sem:IV]), 
   {combine(vp:VP,[iv:IV])}.

vp(vp(V_st,NP_st),[coord:no,inf:I,num:Num,gap:G,sem:VP])-->   
   tv(V_st,[inf:I,num:Num,sem:TV]), 
   np(NP_st,[coord:_,num:_,gap:G,sem:NP]), 
   {combine(vp:VP,[tv:TV,np:NP])}.


/*========================================================================
   Prepositional Phrases
========================================================================*/

pp(pp(P_st,NP_st),[sem:PP])--> 
   prep(P_st,[sem:Prep]), 
   np(NP_st,[coord:_,num:_,gap:[],sem:NP]), 
   {combine(pp:PP,[prep:Prep,np:NP])}.


/*========================================================================
   Relative Clauses
========================================================================*/

rc(rc(Relpro_st,VP_st),[sem:RC])--> 
   relpro(Relpro_st,[sem:RP]), 
   vp(VP_st,[coord:_,inf:fin,num:sg,gap:[],sem:VP]), 
   {combine(rc:RC,[relpro:RP,vp:VP])}.


/*========================================================================
   Lexical Rules
========================================================================*/

iv(v(Leaf),[inf:Inf,num:Num,sem:Sem])--> 
   {lexEntry(iv,[symbol:Sym,syntax:Word,inf:Inf,num:Num])},
   Word,
   {semLex(iv,[symbol:Sym,sem:Sem]),
    extract_leaves(Word,Leaf)}.

tv(v(Leaf),[inf:Inf,num:Num,sem:Sem])--> 
   {lexEntry(tv,[symbol:Sym,syntax:Word,inf:Inf,num:Num])},
   Word,
   {semLex(tv,[symbol:Sym,sem:Sem]),
    extract_leaves(Word,Leaf)}.

cop(aux(Leaf),[inf:Inf,num:Num,sem:Sem])--> 
   {lexEntry(cop,[pol:Pol,syntax:Word,inf:Inf,num:Num])},
   Word,
   {semLex(cop,[pol:Pol,sem:Sem]),
    extract_leaves(Word,Leaf)}.

det(det(Leaf),[mood:M,type:Type,sem:Det])--> 
   {lexEntry(det,[syntax:Word,mood:M,type:Type])},
   Word,
   {semLex(det,[type:Type,sem:Det]),
    extract_leaves(Word,Leaf)}. 

pn(pn(Leaf),[sem:Sem])--> 
   {lexEntry(pn,[symbol:Sym,syntax:Word])},
   Word,  
   {semLex(pn,[symbol:Sym,sem:Sem]),
    extract_leaves(Word,Leaf)}.

relpro(rpro(Leaf),[sem:Sem])--> 
   {lexEntry(relpro,[syntax:Word])},
   Word,
   {semLex(relpro,[sem:Sem]),
    extract_leaves(Word,Leaf)}.

prep(p(Leaf),[sem:Sem])--> 
   {lexEntry(prep,[symbol:Sym,syntax:Word])},
   Word,
   {semLex(prep,[symbol:Sym,sem:Sem]),
    extract_leaves(Word,Leaf)}.

adj(a(Leaf),[sem:Sem])--> 
   {lexEntry(adj,[symbol:Sym,syntax:Word])},
   Word,
   {semLex(adj,[symbol:Sym,sem:Sem]),
    extract_leaves(Word,Leaf)}.

av(aux(Leaf),[inf:Inf,num:Num,sem:Sem])--> 
   {lexEntry(av,[syntax:Word,inf:Inf,num:Num,pol:Pol])},
   Word,
   {semLex(av,[pol:Pol,sem:Sem]),
    extract_leaves(Word,Leaf)}.

coord(Leaf,[type:Type,sem:Sem])--> 
   {lexEntry(coord,[syntax:Word,type:Type])},
   Word, 
   {semLex(coord,[type:Type,sem:Sem]),
    extract_leaves(Word,Leaf)}.

qnp(q_np(Leaf),[mood:M,sem:NP])--> 
   {lexEntry(qnp,[symbol:Symbol,syntax:Word,mood:M,type:Type])},
   Word,
   {semLex(qnp,[type:Type,symbol:Symbol,sem:NP]),
    extract_leaves(Word,Leaf)}.

noun(n(Leaf),[sem:Sem])--> 
   {lexEntry(noun,[symbol:Sym,syntax:Word])},
   Word,
   {semLex(noun,[symbol:Sym,sem:Sem]),
    extract_leaves(Word,Leaf)}.



%---------------------------------------------------------------------------------
% only one string/list-of-numbers conversion in each direction for each string

% extract_leaves(+ListOfStrings,-Single_String_With_Underscores_Between_Components)

extract_leaves([H],H).                           % special case: singleton list

extract_leaves([H|[H2|T]],Res) :-                % nontrivial case: conversion to do
	extract_leaves4([H|[H2|T]],[],[],Res).

extract_leaves4([H|Ts],Acc,[],Res) :-            % accumulate reversed conversion list
	extract_leaves4(Ts,[H|Acc],[],Res).

extract_leaves4([],[H],U_Nums,Res) :-            % last element to prepend reached
	!,
	name(H,HNums),
	appendLists(HNums,U_Nums,Res_Nums),
	name(Res,Res_Nums).

extract_leaves4([],[H|T],List_Nums,Res) :-        % nonempty tail of conversion list
	name(H,HNum),
	appendLists(HNum,List_Nums,NewList_Nums),
	extract_leaves4([],T,[95|NewList_Nums],Res).  % insert underscore
